% Calculate prn positions
clc
clear all
close all

% Universal constants
mu      =   3.986005e14;            % Value of mu

% Orbital parameters
a       =   2.537638e05;            % Semi major axis
ecc     =   4.05141e-01;            % Eccentricity
M0      =   1.04308e-07;            % Initial mean anomaly
omega   =   1.48013e-01;            % Argument of Perigee
i       =   5.92013e-01;            % Inclination
OMEGA   =   5.64831e-01;            % RAAN


disp('The required parameters are:')
disp(['mu      =   Universal)          '  num2str(mu)]);
disp(['a       =   Semi Major Axis     '  num2str(a)]);
disp(['e       =   Eccentricity        '  num2str(ecc)]);
disp(['M0      =   Initial mean anomaly'  num2str(M0)]);
disp(['omega   =   Argument of Perigee '  num2str(omega)]);
disp('.');
disp('Press a key to continue')
disp('.');
disp('.');
pause



% Time span
TS      =   (1:0.1:100);                % In terms of seconds after periapsis

for t = TS(1):TS(end)

    % Derived Parameters

    n       =   sqrt(mu/(a^3));         % Mean angular velocity
    M       =   M0+n*t;                 % Mean anomaly at cuttent instant
    E       =   lsqnonlin(@Efun,M,[],[],[],M,ecc);
                                        % Eccentric Anomaly
    % NOTE:
    % Estimation of Eccentric anomaly from Mean anomaly is done with Nonlinear
    % Least square method. This can be replaced by iterative Newton Raphson
    % method

    r       =   a*(1-ecc*cos(E));                       % radial range

    numer   =   sqrt(1-ecc^2)*sin(E)/(1-ecc*cos(E));    % sin (nu)
    denom   =   (cos(E)-ecc)/(1-ecc*cos(E));            % cos (nu)
    nu      =   atan2(numer,denom);                     % nu

    % NOTE:
    % We have used the expression for the tangent of (nu) instead of only
    % cosine given in the book. This is purely for the cause of convenience
    % as the MATLAB function atan2() provides the solution in four quadrants
    % while acos() or asin()gives only in two quadrants.


    u       =   nu + omega;             % Argument of Latitude


    if t ==TS(1)

        disp('The derived parameters are:')
        disp(['n       =   sqrt(mu/a^3)         '  num2str(n)]);
        disp(['M       =   M0+n*t               '  num2str(M)]);
        disp(['E       =   Derived by iteration '  num2str(E)]);
        disp(['nu      =   Derived from E       '  num2str(nu)]);
        disp(['u       =   nu + omega           '  num2str(u)]);
        disp('.');
        disp('Press a key to continue')
        disp('.');
        disp('.');
        pause

    end



    % Coordinates on Orbital plane

    x1      =   r*cos(u);
    y1      =   r*sin(u);


    if t==TS(1)

        disp('The coordinates on the orbital plane are:')
        disp(['x1 = ' num2str(x1)]);
        disp(['y1 = ' num2str(y1)]);
        disp('.');
        disp('Press a key to continue');
        disp('.');
        disp('.');
        pause

    end


    % Co ordinates on Intermediate frame

    x2= x1;
    y2= y1*cos(i);
    z2= y1*sin(i);


    % Co ordinates in ECI frame

    xi = x2*cos(OMEGA) - y2*sin(OMEGA);
    yi = x2*sin(OMEGA) + y2*cos(OMEGA);
    zi = z2;

    % Enter the position in the sat_info database
    sat_info(t,:) = [xi  yi  zi];


    if t==TS(1)

        disp('The positions in ECI frame are:')
        disp('.');
        disp(['xi =  '  num2str(xi)]);
        disp(['yi =  '  num2str(yi)]);
        disp(['zi =  '  num2str(zi)]);
        disp('.')
        disp('.');
        disp('This is the position of the first instant');
        disp('Similarly positions for subsequent times are determined');
        disp('.');
        disp('Press a key to continue')
        disp('.');
        disp('.');
        pause

    end

end

figure
load coast
load geoid
axesm ('globe', 'Frame', 'on', 'Grid', 'on');
geoshow(geoid, geoidrefvec,'DisplayType','texturemap')
plotm(lat, long)


view(10,60);
axis off
hold on
plot3(sat_info(:,1)/1e5,sat_info(:,2)/1e5,sat_info(:,3)/1e5);